-- Modifying some Static Entity Functions
local size = 66 -- Chip Size
local speed = 700
local snap_distance = 10
local image = love.graphics.newImage('img/chips.png')
local w, h = image:getDimensions()

local shufflechips = love.audio.newSource('sfx/shuffle.wav', 'static')
local movechip = love.audio.newSource('sfx/move.wav', 'static')

local quads = {}
quads['A'] = love.graphics.newQuad(   0,       0, size, size, w, h)
quads['B'] = love.graphics.newQuad(size,       0, size, size, w, h)
quads['C'] = love.graphics.newQuad(size* 2,    0, size, size, w, h)
quads['D'] = love.graphics.newQuad(size* 3,    0, size, size, w, h)
quads['E'] = love.graphics.newQuad(size* 4,    0, size, size, w, h)
quads['F'] = love.graphics.newQuad(size* 5,    0, size, size, w, h)
quads['G'] = love.graphics.newQuad(size* 6,    0, size, size, w, h)
quads['H'] = love.graphics.newQuad(size* 7,    0, size, size, w, h)
quads['I'] = love.graphics.newQuad(size* 8,    0, size, size, w, h)
quads['J'] = love.graphics.newQuad(size* 9,    0, size, size, w, h)
quads['K'] = love.graphics.newQuad(size*10,    0, size, size, w, h)
quads['L'] = love.graphics.newQuad(size*11,    0, size, size, w, h)
quads['M'] = love.graphics.newQuad(size*12,    0, size, size, w, h)
quads['N'] = love.graphics.newQuad(   0,    size, size, size, w, h)
quads['O'] = love.graphics.newQuad(size,    size, size, size, w, h)
quads['P'] = love.graphics.newQuad(size* 2, size, size, size, w, h)
quads['Q'] = love.graphics.newQuad(size* 3, size, size, size, w, h)
quads['R'] = love.graphics.newQuad(size* 4, size, size, size, w, h)
quads['S'] = love.graphics.newQuad(size* 5, size, size, size, w, h)
quads['T'] = love.graphics.newQuad(size* 6, size, size, size, w, h)
quads['U'] = love.graphics.newQuad(size* 7, size, size, size, w, h)
quads['V'] = love.graphics.newQuad(size* 8, size, size, size, w, h)
quads['W'] = love.graphics.newQuad(size* 9, size, size, size, w, h)
quads['X'] = love.graphics.newQuad(size*10, size, size, size, w, h)
quads['Y'] = love.graphics.newQuad(size*11, size, size, size, w, h)
quads['Z'] = love.graphics.newQuad(size*12, size, size, size, w, h)

local function newChip(quad_tile, x, y)
  local chip = newStaticEntity(image, love.graphics.getWidth() / 2, y)
  chip.char = quad_tile
  chip.quad = quads[quad_tile]
  chip.tx = x
  chip.ty = y
  chip.width = size
  chip.height = size
  chip.ox = size / 2
  chip.oy = size / 2
  chip.tray = 'chips'
  
  function chip.update(dt)
    if getCircleCircleDistance(chip.x, chip.y, size / 2, chip.tx, chip.ty) + snap_distance <= 0 then
      chip.x = chip.tx
      chip.y = chip.ty
    else
      chip.x = chip.x + (chip.vx * dt)
      chip.y = chip.y + (chip.vy * dt)
    end
  end
  function chip.draw()
    love.graphics.draw(image, chip.quad, chip.x, chip.y, chip.r, chip.sx, chip.sy, chip.ox, chip.oy)
  end
  
  function chip.updateVelocity()
    local angle = getAngle(chip.x, chip.y, chip.tx, chip.ty)
    chip.vx, chip.vy = getVelocityXY(speed, angle)
  end
  
  return chip
end

local function newChips(word, indexes)
  local chips = {}
  
  for i = 1, word:len() do
    table.insert(chips, newChip(word:sub(i, i), indexes[i].x, indexes[i].y))
  end
  
  return chips
end

function newChipMachine(word, tray1, tray2)
  local machine = {}
  
  machine.chips = newChips(word, tray2)
  
  function machine.update(dt)
    for _, v in pairs(machine.chips) do
      v.update(dt)
    end
  end
  
  function machine.draw()
  
    love.graphics.setColor(0, 100/255, 100/255, 0.3)
    love.graphics.rectangle('fill',
      tray2[1].x - (size / 2) - 5,
      tray2[1].y - (size / 2) - 5,
      (size * word:len()) + 8,
      size + 8,
      5, 5
    )
    
    love.graphics.setColor(1, 1, 1)
    for _, v in pairs(machine.chips) do
      v.draw()
    end
  end
  
  function machine.char_press(key)
    local index = 0
    for i, v in pairs(machine.chips) do
      if v.tray == 'chips' and string.upper(key) == v.char then
        index = i
      end
    end
    
    if index ~= 0 then
      machine.toTray(index, 'chips')
    end
  end
  
  function machine.type_word(word)
    if word:len() > 0 then
      for i = 1, word:len() do
        machine.char_press(word:sub(i, i))
      end
    end
  end
  
  function machine.enter()
    local letters = {}
    local word = ''
    
    -- Getting each character
    for _, v in pairs(machine.chips) do
      if v.tray == 'guess' then
        table.insert(letters, {char = v.char, tx = v.tx})
      end
    end

    if #letters > 1 then
      -- Sort from lowest tx
      for a = 1, #letters - 1 do
        for b = a+1, #letters do
          if letters[a].tx > letters[b].tx then
            local t = letters[a]
            letters[a] = letters[b]
            letters[b] = t
          end
        end
      end
    end
    
    -- then concatenate them all
    for a = 1, #letters do
      word = word .. letters[a].char
    end
    
    -- and return produced word
    return word
  end
  
  function machine.shuffle()
    -- Shuffle Only from Chips Tray
    local shuffle_indexes = {}
    for _, v in pairs(machine.chips) do
      if v.tray == 'chips' then
        table.insert(shuffle_indexes, v.tx)
      end
    end
    
    for _, v in pairs(machine.chips) do
      if v.tray == 'chips' then
        local rx = love.math.random(#shuffle_indexes)
        v.tx = shuffle_indexes[rx]
        table.remove(shuffle_indexes, rx)
        v.updateVelocity(v.tx, v.ty)        
      end
    end
    
    shufflechips:stop()
    shufflechips:play()
  end
  
  function machine.toTray(index, current_tray)
    local target_tray = current_tray == 'chips' and 'guess' or 'chips'
    local target_y    = current_tray == 'chips' and tray1[1].y or tray2[1].y
    
    -- Find the smallest Unoccupied tx in the target tray
    local hole = false

    --From Chips = To First, From Guess = To Last
    local mark = current_tray == 'chips' and 0 or #tray1 + 1
    local step = current_tray == 'chips' and 1 or -1
    
    while not hole do
      hole = true
      mark = mark + step
      for _, v in pairs(machine.chips) do
        if v.tray == target_tray and v.tx == tray1[mark].x then
          hole = false
        end
      end
    end
    
    local target_x = tray1[mark].x
    
    machine.chips[index].tray = target_tray
    machine.chips[index].tx = target_x
    machine.chips[index].ty = target_y
    machine.chips[index].updateVelocity()
    
    movechip:stop()
    movechip:play()
  end

  -- A simplified form of toTray  
  function machine.backspace()
    -- First, see if a chip is actually in tray1
    local chip = 0
    local x = tray1[1].x
    
    for i, v in pairs(machine.chips) do
      if v.tray == 'guess' then
        -- Now see if it's the last object on the tray
        if v.tx >= x then
          chip = i
          x = v.tx
        end
      end
    end
    
    -- Looking for a hole from the Last Chips Tray
    if chip > 0 then
      local hole = false
      local mark = #tray2 + 1
      
      while hole == false do
        hole = true
        mark = mark - 1
        for _, v in pairs(machine.chips) do
          if v.tray == 'chips' and v.tx == tray2[mark].x then
            hole = false
          end
        end
      end

      machine.chips[chip].tray = 'chips'
      machine.chips[chip].tx = tray2[mark].x
      machine.chips[chip].ty = tray2[mark].y
      machine.chips[chip].updateVelocity()
      
      movechip:stop()
      movechip:play()
    end
  end

  -- And even simpler  
  function machine.clear()
    local counter = 0

    for _, v in pairs(machine.chips) do
      if v.tray == 'guess' then
        counter = counter + 1
      end
    end
    
    -- I sensed a bug if we don't count first
    for i = 1, counter do
      machine.backspace()
    end
  end
  
  return machine
end
