dictionary = {}

local font = love.graphics.newImageFont('img/font_wordlist.png','ABCDEFGHIJKLMNOPQRSTUVWXYZ ', -2)

function dictionary.getLongestWord(minlength)
  local length = minlength or 6

  -- Gather all the long words
  local list = {}
  for word in love.filesystem.lines('file/wordlist.dat') do
    if word:len() >= minlength then -- Words with at least <length> letters
      table.insert(list, word)
    end
  end

  -- Choose one from that list  
  return list[love.math.random(#list)]
end

function dictionary.searchSubwords(word)
  local list = {}
  
  local rows = 13 -- Number of rows for each Column
  local maxwidth = 0 -- Adds for every row
  local y = 5 -- Starting y
  local rowspaces = 4 -- I think for every column & row
  
  local temp_y = 0
  local colspaces = 10
  local tempcolwidth = 0
  
  -- Searching for possible subwords
  for subword in love.filesystem.lines('file/wordlist.dat') do

    -- Put each letter on a table
    local used_chars = {}
    for a = 1, subword:len() do
      table.insert(used_chars, string.sub(subword, a, a))
    end

    -- c: checks all of the character of the given word
    -- d: iterates over all the characters in used_chars
    local b = 1
    while #used_chars > 0 and b <= #word do
      local removed = false
      
      -- If word's character is found in used_chars, then remove it
      local c = 1
      while c <= #used_chars and removed == false do
        if used_chars[c] == string.sub(word, b, b) then
          table.remove(used_chars, c)
          removed = true
        else
          c = c + 1
        end
      end
      b = b + 1
    end
    
    -- if used_chars has used up all its letters, then it's a subword
    if #used_chars == 0 then
      table.insert(list, {word = subword,
        x = maxwidth + colspaces,
        y = y + temp_y * (font:getHeight(subword) + rowspaces),
        solved = false}
      )
      
      colwidth = font:getWidth(subword)
      temp_y = temp_y + 1
      
      if temp_y == rows then
        temp_y = 0
        maxwidth = maxwidth + colwidth + colspaces
      end
    end
  end
  
  if temp_y > 0 then
    maxwidth = maxwidth + colwidth + colspaces
  end

  dictionary.width = maxwidth + colspaces
  dictionary.height = rows * (font:getHeight('W') + rowspaces) + (rowspaces * 4)
  dictionary.x = (love.graphics.getWidth() / 2) - (dictionary.width / 2)
  dictionary.y = 0 - (rowspaces * 2)
  
  return list
end

function dictionary.searchMatch(word, list)
  local index = nil
  for i, v in pairs(list) do
    if v.word == word then
      index = i
    end
  end
  
  return index
end

function dictionary.draw(subwords, finished)
  --Subword Tray
  love.graphics.setColor(0, 0, 1, 0.2)
  love.graphics.rectangle('fill', dictionary.x, dictionary.y, dictionary.width, dictionary.height, 4, 4)
  
  --Subwords
  love.graphics.setFont(font)
  love.graphics.setColor(1, 1, 1)
  for _, v in pairs(subwords) do
    if v.solved then
      love.graphics.print({{1, 1, 1}, v.word}, dictionary.x + v.x, v.y)
    else
      if finished then
        love.graphics.print({{1,1,0}, v.word}, dictionary.x + v.x, v.y)
      else
        local word = ''
        for i = 1, v.word:len() do
          word = word .. ' '
        end
        love.graphics.print({{1, 1, 1}, word}, dictionary.x + v.x, v.y)
      end
    end
  end
end
