-- TASK: MUST SORT FROM HIGHEST SCORE

state_highscores = newState({0, 175/255, 175/255})

local title_font = love.graphics.newFont(50)
local title_text = 'HIGH SCORES'

local list_font = love.graphics.newFont(35)

local menu_font = love.graphics.newFont(30)
local menu = newMenu(menu_font, 4, nil, {50/255, 50/255, 50/255, 0.5})
local see_newest = false

menu.addButton('BACK', love.graphics.getWidth() / 2, 550,
  function()
    for _, v in pairs(highscores) do
      v.status = 'old'
    end
    state_machine.change(state_title)
  end
)

-- I will only fit what I can draw
-- For now, 10 is the ideal number
function sort_highscores()
  if #highscores > 1 then
    for champion = 1, #highscores - 1 do
      for challenger = champion + 1, #highscores do
        if highscores[champion].score < highscores[challenger].score then
          local t_name = highscores[champion].name
          local t_score = highscores[champion].score
          local t_status = highscores[champion].status

          highscores[champion].name = highscores[challenger].name
          highscores[champion].score = highscores[challenger].score
          highscores[champion].status = highscores[challenger].status

          highscores[challenger].name = t_name
          highscores[challenger].score = t_score
          highscores[challenger].status = t_status
        end
      end
    end
  end
end

function save_highscores()
  local text = ''
  for _, v in pairs(highscores) do
    text = text .. v.name .. ' ' .. v.score .. ' ' .. v.status .. '\n'
  end
  love.filesystem.write('highscores.lst', text)
end

function load_highscores()
  highscores = {}
  -- This file MUST EXIST
  if love.filesystem.getInfo('highscores.lst') then
    for sentence in love.filesystem.lines('highscores.lst') do
      local words = {}
    
      -- %s+ whitespace
      -- %S+ not whitespace
      for word in sentence:gmatch('%S+') do
        table.insert(words, word)
      end
    
      -- 1st Word: Name
      -- 2nd Word: Score (Converted to Number)
      -- 3rd Word: Status (Old or New Entry) but I'll turn everyone to 'old'
      table.insert(highscores, { name = words[1], score = tonumber(words[2]), status = 'old'})
    end
  else
    for i = 1, 10 do
      table.insert(highscores, {name = 'Anonymous', score = i * 100, status = 'old'})
    end
  end
  
  sort_highscores()
end

function state_highscores.load(params)
  see_newest = params or false
end

function state_highscores.draw()
  love.graphics.setFont(title_font)
  love.graphics.print(title_text, (love.graphics.getWidth() / 2) - (title_font:getWidth(title_text) / 2), 10)

  if #highscores > 0 then
  love.graphics.setFont(list_font)
    for i, v in pairs(highscores) do
      local y = 100 + (i - 1) * 40
      if see_newest and v.status == 'new' then
        love.graphics.setColor(1,1,0)
      else
        love.graphics.setColor(state_highscores.defcolor)
      end
      love.graphics.print(v.name, 150, y)
      love.graphics.printf(v.score, 0, y, love.graphics.getWidth() - 150, 'right')
    end
  end
  
  menu.draw()
end

function state_highscores.keypressed(key)
  if key == 'return' or key == 'space' then
    menu.button_press()
  else
    menu.cycle_up()
  end
end

function state_highscores.keyreleased(key)
  if key == 'return' or key == 'space' then
    menu.button_release()
  else
    menu.cycle_up()
  end
end

function state_highscores.mousemoved(x, y, dx, dy)
  menu.mouse_over(x, y)
end
function state_highscores.mousepressed(x, y, button)
  if button == 1 then menu.button_press() end
end
function state_highscores.mousereleased(x, y, button)
  if button == 1 then menu.button_release() end
end
