state_play = newState()

local background_image = love.graphics.newImage('img/background.png')
local misc_sprites = love.graphics.newImage('img/misc.png')
local longest_word_length = 6

local font = love.graphics.newFont(23)
local score = 0
local score_x = 80

local word_to_guess = ''
local subwords = {}
local guess_tray = {}
local chips_tray = {}
local buttons = {}
local chip_machine = {}
local word_whacked = false
local timer_over = false

local backspace = {
  pressed = false,
  counter = 0,
}

local entered_word = ''
local previous_word = ''

local error_sound = love.audio.newSource('sfx/error.wav', 'static')
local found_sound = love.audio.newSource('sfx/found.wav', 'static')
local win = love.audio.newSource('sfx/win.wav', 'static')

local function enterWord()
  entered_word = chip_machine.enter()

  if entered_word:len() < 3 then
    message.change('Please put at least 3 letters.')
    error_sound:stop()
    error_sound:play()
  else
    local index = dictionary.searchMatch(entered_word, subwords)

    if index then -- Subword Found
    
      if subwords[index].solved then -- If it has already been found
        message.change("'" .. entered_word .. "' has already been found.")
        error_sound:stop()
        error_sound:play()
      else
        subwords[index].solved = true
        score = score + ((entered_word:len() ^ 2) * 10)
        
        -- Check first if all words have been found
        local all_solved = true
        for _, v in pairs(subwords) do
          if not v.solved then
            all_solved = false
          end
        end
        
        -- If all words have been found
        if all_solved then
          state_machine.push(state_advance, {all_solved = all_solved, score = score})
          win:stop()
          win:play()
        -- If it's the longest word
        elseif entered_word:len() == subwords[#subwords].word:len() and not word_whacked then
          word_whacked = true
          message.change('WORD WHACKED!')
          control.addAdvanceLevelButton()
          control.buttons[7].fn = function() state_machine.push(state_advance, {score = score, all_solved = all_solved}) end         
          
          win:stop()
          win:play()
        
        else
          found_sound:stop()
          found_sound:play()
        end
      end
    else -- Subword Not Found
      message.change("'" .. entered_word .. "' does not exist in the dictionary.")
      error_sound:stop()
      error_sound:play()
    end

    previous_word = entered_word
    chip_machine.clear()
  end
end

function state_play.load(params)
  score = params or 0
  word_to_guess = dictionary.getLongestWord(longest_word_length)
  subwords = dictionary.searchSubwords(word_to_guess)
  guess_tray, chips_tray = newGuessTray(word_to_guess:len(), misc_sprites) -- Tray uses same sprite as Game Buttons
  control = newGameButtons(misc_sprites)
  chip_machine = newChipMachine(word_to_guess, guess_tray.indexes, chips_tray)
  chip_machine.shuffle()

  control.buttons[1].fn = function() enterWord() end
  control.buttons[2].fn = function() chip_machine.shuffle() end
  control.buttons[3].fn = function() chip_machine.clear() end
  control.buttons[4].fn = function() chip_machine.clear() chip_machine.type_word(previous_word) end
  control.buttons[5].fn = function() state_machine.push(state_pause, true) end
  control.buttons[6].fn = function() state_machine.push(state_pause) end

  entered_word = ''
  previous_word = ''
  word_whacked = false
  time_over = false
  timer.initialize()
  message.change('')
end

function state_play.update(dt)
  if backspace.pressed == true then
    backspace.counter = backspace.counter + dt
    if backspace.counter >= 0.6 then
      backspace.pressed = false
      backspace.counter = 0
      chip_machine.clear()
    end
  end
  
  time_over = timer.update(dt)
  if time_over then
    message.change('')
    state_machine.push(state_timeover, {score = score, word_whacked = word_whacked})
  end
  chip_machine.update(dt)
  message.update(dt)
end

function state_play.draw()
  love.graphics.draw(background_image)

  -- Can create 15 things in a column
  dictionary.draw(subwords, time_over)
  guess_tray.draw()
  control.draw()
  timer.draw()
  message.draw()
  chip_machine.draw()
  love.graphics.setFont(font)
  love.graphics.print('SCORE:', score_x - (font:getWidth('SCORE:') / 2), 515)
  love.graphics.print(score, score_x - (font:getWidth('' .. score) / 2), 540)
end

function state_play.mousepressed(x, y, button)
  if button == 1 then
    for i, v in pairs(chip_machine.chips) do
      if x >= v.x - (v.width / 2) and x <= v.x + (v.width / 2) then
        if y >= v.y - (v.height / 2) and y <= v.y + (v.height / 2) then
          chip_machine.toTray(i, v.tray, guess_tray.indexes, chips_tray)
        end
      end
    end
    control.button_press()
  end
end

function state_play.keypressed(key)
  if key == 'return' then
    -- Check if there are chips on the Guess Tray
    local empty = true
    for _, v in pairs(chip_machine.chips) do
      if v.tray == 'guess' then
        empty = false
      end
    end
    if empty then
      chip_machine.type_word(previous_word)
    else
      enterWord()
    end
  elseif key == 'space' then
    chip_machine.shuffle()
  elseif key == 'backspace' then
    backspace.pressed = true
  else
    chip_machine.char_press(key)
  end
end
function state_play.keyreleased(key)
  if key == 'backspace' then
    if backspace.pressed then
      backspace.pressed = false
      backspace.counter = 0
      chip_machine.backspace()
    end
  end
end
function state_play.mousereleased(x, y, button)
  if button == 1 then
    control.button_release()
  end
end
function state_play.mousemoved(x, y, dx, dy)
  control.mouse_over(x, y)
end
