state_timeover = newState({75/255, 75/255, 75/255, 0.35}, {175, 175/255, 0})

local title_y = 283
local title_font = love.graphics.newFont(80)
local title = ''

local message_y = 362
local message_color = {1, 0, 0}
local message_font = love.graphics.newFont(30)
local message_nwidth = 150 -- Negative Width from screen width
local message = ''

local input = nil
local input_font = love.graphics.newFont('font/Bergen Mono Regular.otf', 75)
local input_padding = 5
local input_x = (love.graphics.getWidth() / 2) - (input_font:getWidth('WWWWWWWWWW') / 2)
local input_y = 410

local menu = {}
local menu_y = 540
local menu_font = love.graphics.newFont(40)

local highscore = false
local score = 0

local timesound = love.audio.newSource('sfx/timer_end.wav', 'static')

local function insert_highscore(name)
  highscores[#highscores].name = name
  highscores[#highscores].score = score
  highscores[#highscores].status = 'new'
  sort_highscores()
  save_highscores()
  state_machine.change(state_highscores, true)
end

function state_timeover.load(params) -- 'Game Over' or 'Advance to Next Level'
  title = "TIME'S UP"
  menu = newMenu(menu_font, 3, state_timeover.defcolor, {0, 0, 100/255, 0.68}, {100/255, 50/255, 0})
  input = nil
  score = params.score

  if params.word_whacked then
    message = 'You managed to Whack a Word! Get ready for the next set.'
    menu.addButton('NEXT WORD', love.graphics.getWidth() / 2, menu_y, function() state_machine.change(state_play, params.score) end)
  else
    -- Check first if you've managed to get a highscore
    highscore = false
    for _,v in pairs(highscores) do
      if v.score < params.score then
        highscore = true
      end
    end
    
    if highscore then
      message = 'You got a High Score!'
      input = newTextInput(input_font, input_x, input_y, 10, 'left', input_padding)
      menu.addButton('ENTER NAME', love.graphics.getWidth() / 2, menu_y,
        function()
          insert_highscore(input.text)
        end
      )
    else
      message = 'You can do better next time.'
      menu.addButton('PLAY AGAIN', 275, menu_y, function() state_machine.change(state_play) end)
      menu.addButton('EXIT', 525, menu_y, function() state_machine.change(state_title) end)
    end
  end
  
  timesound:stop()
  timesound:play()
end

function state_timeover.update(dt)
  if input then
    input.update(dt)
  end
end

function state_timeover.draw()
  love.graphics.setColor(state_timeover.bgcolor)
  love.graphics.rectangle('fill', 0, 0, love.graphics.getWidth(), love.graphics.getHeight())

    local w = 300
    
    love.graphics.setColor(0, 0, 0, 0.5)
    love.graphics.rectangle('fill', (love.graphics.getWidth() / 2) - w, title_y - 10, w * 2, 310, 10, 10)

  love.graphics.setColor(state_timeover.defcolor)
  love.graphics.setFont(title_font)
  love.graphics.print(title, (love.graphics.getWidth() / 2) - (title_font:getWidth(title) / 2), title_y)

  love.graphics.setColor(message_color)
  love.graphics.setFont(message_font)
  love.graphics.printf(message, message_nwidth, message_y, love.graphics.getWidth() - (message_nwidth * 2), 'center')
  
  if input then
    input.draw()
  end

  menu.draw()
end

function state_timeover.keypressed(key)
  if input then
    input.keypressed(key)
    if key == 'return' then
      if input.text:len() > 1 then
        insert_highscore(input.text)
      end
    end
  else
    if key == 'return' or key == 'space' then
      menu.button_press()
    elseif key == 'left' or 'up' then
      menu.cycle_up()
    elseif key == 'right' or 'down' then
      menu.cycle_down()
    end
  end
end
function state_timeover.keyreleased(key)
  if key == 'enter' or key == 'space' and not input then
    menu.button_release()
  end
end

function state_timeover.textinput(t)
  if input then
    input.textinput(t)
  end
end

function state_timeover.mousemoved(x, y, dx, dy)
  menu.mouse_over(x, y)
end
function state_timeover.mousepressed(x, y, button)
  menu.button_press()
end
function state_timeover.mousereleased(x, y, button)
  menu.button_release()
end
